' Copyright 2008 Jozef Vince <jozef.vince@freemap.sk>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

' TODO:for dodi ~ has to be rewritten in better way because current is ugly 
''' <summary>
''' Battery Status diplay class
''' </summary>
Public Class BatteryStatus
    ''' <summary>
    ''' AC line status enumeration
    ''' </summary>
    Public Enum ACLineStatus As Byte
        ''' <summary>
        ''' on batteries
        ''' </summary>
        AC_LINE_OFFLINE = &H0
        ''' <summary>
        ''' on AC line
        ''' </summary>
        AC_LINE_ONLINE = &H1
        ''' <summary>
        ''' runiing from backup power source
        ''' </summary>
        AC_LINE_BACKUP_POWER = &H2
        ''' <summary>
        ''' unknown status (device emulator report this)
        ''' </summary>
        AC_LINE_UNKNOWN = &HFF
    End Enum

    ''' <summary>
    ''' Battery status flags
    ''' </summary>
    Public Enum BatteryFlags As Byte
        ''' <summary>
        ''' High
        ''' </summary>
        BATTERY_FLAG_HIGH = &H1
        ''' <summary>
        ''' Low power
        ''' </summary>
        BATTERY_FLAG_LOW = &H2
        ''' <summary>
        ''' critical low power
        ''' </summary>
        BATTERY_FLAG_CRITICAL = &H4
        ''' <summary>
        ''' charging
        ''' </summary>
        BATTERY_FLAG_CHARGING = &H8
        ''' <summary>
        ''' device has no batteries
        ''' </summary>
        ATTERY_FLAG_NO_BATTERY = &H80
        ''' <summary>
        ''' unknown
        ''' </summary>
        BATTERY_FLAG_UNKNOWN = &HFF
        ''' <summary>
        ''' same as unknown
        ''' </summary>
        BATTERY_PERCENTAGE_UNKNOWN = &HFF
    End Enum

    Private Class SYSTEM_POWER_STATUS_EX
        Public ACLineStatus As ACLineStatus
        Public BatteryFlag As BatteryFlags
        Public BatteryLifePercent As Byte
        Public Reserved1 As Byte
        <CLSCompliant(False)> _
        Public BatteryLifeTime As System.UInt32
        <CLSCompliant(False)> _
        Public BatteryFullLifeTime As System.UInt32
        Public Reserved2 As Byte
        Public BackupBatteryFlag As BatteryFlags
        Public BackupBatteryLifePercent As Byte
        Public Reserved3 As Byte
        <CLSCompliant(False)> _
        Public BackupBatteryLifeTime As System.UInt32
        <CLSCompliant(False)> _
        Public BackupBatteryFullLifeTime As System.UInt32
    End Class

    ''' <summary>
    '''  battery metering class
    ''' </summary>
    Private Class BatteryMeter
        ''' <summary>
        ''' time of first check
        ''' </summary>
        Public FirstCheck As Date
        ''' <summary>
        ''' battery power percentage whe first checked
        ''' </summary>
        Public FirstPercentage As Byte
        ''' <summary>
        ''' time of last percentage check
        ''' </summary>
        Public CurrentCheck As Date
        ''' <summary>
        ''' current power percentage
        ''' </summary>
        Public CurrentPercentage As Byte
        ''' <summary>
        ''' time in seconds since on batteries (or OSMtracker is started)
        ''' </summary>
        Public TimeRun As Integer
        ''' <summary>
        ''' Estimated remaining time until batteries a dried out
        ''' </summary>
        Public TimeRemaining As Integer
        ''' <summary>
        ''' date/time when each level of power percentge is reached
        ''' </summary>
        Public CheckHistory(101) As Date

        ''' <summary>
        ''' running on batteries ?
        ''' </summary>
        Private runingOnBatteries As Boolean = False
        ''' <summary>
        ''' currently charging
        ''' </summary>
        Private batteryCharging As Boolean = False

        ''' <summary>
        ''' Called when device is on [ac line].
        ''' </summary>
        ''' <param name="percentage">The percentage.</param>
        ''' <param name="batteryFlag">The battery flag.</param>
        Public Sub OnAcLine(ByVal percentage As Byte, ByVal batteryFlag As BatteryFlags)
            If runingOnBatteries = True Then
                runingOnBatteries = False
            End If

            If batteryFlag <> BatteryFlags.BATTERY_FLAG_UNKNOWN Then
                If (batteryFlag And BatteryFlags.BATTERY_FLAG_CHARGING) = BatteryFlags.BATTERY_FLAG_CHARGING Then
                    batteryCharging = True
                    If percentage < 101 Then
                        CurrentPercentage = percentage
                    Else
                        CurrentPercentage = 100
                    End If
                Else
                    batteryCharging = False
                End If
            End If
        End Sub

        ''' <summary>
        ''' Called when device is running on [batteries].
        ''' </summary>
        ''' <param name="percentage">The percentage.</param>
        ''' <param name="batteryFlag">The battery flag.</param>
        Public Sub OnBattery(ByVal percentage As Byte, ByVal batteryFlag As BatteryFlags)
            If runingOnBatteries = False Then
                runingOnBatteries = True
                batteryCharging = False
                If percentage < 101 Then
                    FirstPercentage = percentage
                Else
                    FirstPercentage = 100
                End If
                FirstCheck = Now()
                CheckHistory(FirstPercentage) = Now()
                CurrentCheck = FirstCheck
                CurrentPercentage = FirstPercentage
                TimeRun = 0
                TimeRemaining = -1
            Else
                If percentage > 100 Then
                    percentage = 100
                End If

                If CurrentPercentage <> percentage Then
                    CurrentPercentage = percentage
                    CheckHistory(CurrentPercentage) = Now()
                    CurrentCheck = Now()
                    TimeRun = CInt(CurrentCheck.Subtract(FirstCheck).TotalSeconds)

                    If CurrentPercentage < FirstPercentage AndAlso TimeRun > 0 Then
                        TimeRemaining = CInt(100.0 / ((FirstPercentage - CurrentPercentage) / TimeRun) - TimeRun)
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' Gets the status string.
        ''' </summary>
        ''' <returns>returns formaated status string</returns>
        Public Function GetStatusString() As String
            GetStatusString = "UNKN"
            If runingOnBatteries = False Then
                If batteryCharging = True Then
                    GetStatusString = "+" & String.Format("{0}%", CurrentPercentage)
                Else
                    GetStatusString = "=100%"
                End If
            Else
                If TimeRemaining > 0 Then
                    Dim t As TimeSpan = TimeSpan.FromSeconds(TimeRemaining)
                    GetStatusString = "~" & (t.Hours).ToString("0") & ":" & (t.Minutes).ToString("00") & "h"
                Else
                    GetStatusString = "~" & String.Format("{0}%", CurrentPercentage)
                End If
            End If
        End Function
    End Class

    <Runtime.InteropServices.DllImport("coredll")> _
    Private Shared Function GetSystemPowerStatusEx(ByVal lpSystemPowerStatus As SYSTEM_POWER_STATUS_EX, ByVal fUpdate As Boolean) As System.UInt32
    End Function

    Private Status As New SYSTEM_POWER_STATUS_EX
    ''' <summary>
    ''' time when the status was checked last time
    ''' </summary>
    Private lastChecked As Date
    ''' <summary>
    ''' True if API can get the SystemPowerStatus
    ''' </summary>
    Private statusValid As Boolean = False

    ''' <summary>
    ''' battery meter for devices battery
    ''' </summary>
    Private deviceBattery As New BatteryMeter

    ''' <summary>
    ''' Gets the AC line status.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetACLineStatus() As ACLineStatus
        CheckStatus()
        If statusValid = True Then
            GetACLineStatus = Status.ACLineStatus
        Else
            GetACLineStatus = ACLineStatus.AC_LINE_UNKNOWN
        End If
    End Function

    ''' <summary>
    ''' Gets the battery flag.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetBatteryFlag() As BatteryFlags
        CheckStatus()
        If statusValid = True Then
            GetBatteryFlag = Status.BatteryFlag
        Else
            GetBatteryFlag = BatteryFlags.BATTERY_FLAG_UNKNOWN
        End If
    End Function

    ''' <summary>
    ''' Gets the battery life percent.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetBatteryLifePercent() As Integer
        CheckStatus()
        If statusValid = True Then
            GetBatteryLifePercent = Status.BatteryLifePercent
        Else
            GetBatteryLifePercent = 255
        End If
    End Function

    ''' <summary>
    ''' Gets the battery status string.
    ''' </summary>
    ''' <returns></returns>
    Public Function GetBatteryStatusString() As String
        CheckStatus()
        If statusValid = True Then
            GetBatteryStatusString = deviceBattery.GetStatusString()
        Else
            GetBatteryStatusString = "UNKN"
        End If
    End Function

    ''' <summary>
    ''' Checks the status.
    ''' </summary>
    ''' <returns></returns>
    Private Function CheckStatus() As Boolean
        CheckStatus = False
        If Now.Subtract(lastChecked).TotalSeconds >= 5 Then
            lastChecked = Now()
            CheckStatus = True
            GetStatus()
        End If
    End Function

    ''' <summary>
    ''' Gets the status.
    ''' </summary>
    Private Sub GetStatus()
        lastChecked = Now()
        If Convert.ToInt32(GetSystemPowerStatusEx(Status, False)) = 1 Then
            statusValid = True
            If (Status.ACLineStatus And ACLineStatus.AC_LINE_ONLINE) = ACLineStatus.AC_LINE_ONLINE Then
                deviceBattery.OnAcLine(Status.BatteryLifePercent, Status.BatteryFlag)
            Else
                deviceBattery.OnBattery(Status.BatteryLifePercent, Status.BatteryFlag)
            End If
        Else
            statusValid = False
        End If
    End Sub

    Public Sub New()
        GetStatus()
    End Sub
End Class
