' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
' based on code by:
' Copyright 2007 Rubke <interrubke@gmail.com>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO.Ports
Imports System.Threading

Namespace GPS
    Public Class DotNetSerialPort
        Inherits SerialPort

        Private WithEvents serialPort1 As New System.IO.Ports.SerialPort

        Public Sub New(ByRef parser As NMEAParser, ByVal commName As String, ByVal baudrate As Integer)
            MyBase.New(parser, commName, baudrate)
        End Sub
        Public Overrides Function Connect() As Boolean
            If commName = "" Or baudRate = 0 Then
                MsgBox("invalid com params", MsgBoxStyle.Critical)
                Return False
            End If

            If serialPort1 IsNot Nothing Then
                If serialPort1.IsOpen Then
                    serialPort1.Close() ': Exit Function
                    Thread.Sleep(200)
                End If
            End If

            ' create new com port
            serialPort1 = New System.IO.Ports.SerialPort(commName, baudRate, Parity.None, 8, StopBits.One)
            If serialPort1 IsNot Nothing Then
                With serialPort1
                    Try
                        .Open()
                        .DiscardInBuffer()

                    Catch ex As Exception
                        '    MsgBox("error opening port, reason : " & ex.ToString, MsgBoxStyle.Critical)
                    End Try
                    If Not .IsOpen Then
                        Connect = False
                        gpsConnected = False
                        MsgBox("Error opening com port", MsgBoxStyle.Critical)
                    Else
                        MyBase.Connect()
                    End If
                End With
            Else
                Connect = False
                gpsConnected = False
                MsgBox("Error opening gps com port", MsgBoxStyle.Critical)
            End If
        End Function
        Public Overrides Sub Disconnect()
            If serialPort1 IsNot Nothing Then
                If serialPort1.IsOpen Then
                    Try
                        serialPort1.DiscardInBuffer()
                        Dim CloseDown As New Thread(New ThreadStart(AddressOf Me.CloseSerial)) ' //close port in new thread to avoid hang
                        CloseDown.Start() ' //close port in new thread to avoid hang
                    Catch ex As Exception
                    End Try
                End If
            End If
            MyBase.Disconnect()
        End Sub
        Private Sub CloseSerial()
            Try
                serialPort1.Close() ' //close the serial port
            Catch ex As Exception
                MessageBox.Show(ex.Message) '; //catch any serial port closing error messages
            End Try
            serialPort1 = Nothing
        End Sub
        Private Sub serialPort1_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles serialPort1.DataReceived
            Dim allData As String = ""
            lastDataReceivedDate = Now
            Try
                If serialPort1.IsOpen Then
                    While serialPort1.BytesToRead <> 0
                        If Not serialPort1.IsOpen Then Exit Sub
                        Dim strdata As String = serialPort1.ReadLine
                        ReceivedData(strdata)
                        allData += strdata
                    End While
                    LogNMEA(allData)
                End If
            Catch ex As Exception
            End Try
        End Sub
        Protected Overrides Sub Finalize()
            Try
                If serialPort1 IsNot Nothing Then
                    If serialPort1.IsOpen Then
                        serialPort1.Close()
                    End If
                    serialPort1 = Nothing
                End If
            Catch ex As Exception
            End Try
            MyBase.Finalize()
        End Sub
    End Class
End Namespace
