' Copyright 2008 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Threading
Imports System.Runtime.InteropServices

Namespace GPS
    Public Class IntermediateDriver
        Inherits GPSDevice

        <DllImport("gpsapi.dll", EntryPoint:="GPSOpenDevice", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
        Private Shared Function GPSOpenDevice(ByVal hNewLocationData As IntPtr, ByVal hDeviceStateChange As IntPtr, ByVal szDeviceName As String, ByVal dwFlags As Integer) As IntPtr
        End Function
        <DllImport("gpsapi.dll", EntryPoint:="GPSCloseDevice", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
        Private Shared Function GPSCloseDevice(ByVal hGPSDevice As IntPtr) As Integer
        End Function
        <DllImport("coredll.dll", EntryPoint:="CloseHandle", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
        Private Shared Function CloseHandle(ByVal hObject As IntPtr) As Integer
        End Function
        <DllImport("gpsapi.dll", EntryPoint:="GPSGetPosition", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
        Private Shared Function GPSGetPosition(ByVal hGPSDevice As IntPtr, ByVal pGPSPosition As IntPtr, ByVal dwMaximumAge As Integer, ByVal dwFlags As Integer) As Integer
        End Function
        Private Structure SatelliteArray
            Public s0, s1, s2, s3, a4, s5, s6, s7, s8, s9, s10, s11 As Integer
        End Structure
        'http://msdn.microsoft.com/en-us/library/bb202062.aspx
        Private Structure _GPS_POSITION
            Public dwVersion As Integer
            Public dwSize As Integer
            Public dwValidFields As Integer
            Public dwFlags As Integer
            Public stUTCTime As SYSTEMTIME
            Public dblLatitude As Double
            Public dblLongitude As Double
            Public flSpeed As Single
            Public flHeading As Single
            Public dblMagneticVariation As Double
            Public flAltitudeWRTSeaLevel As Single
            Public flAltitudeWRTEllipsoid As Single
            Public FixQuality As Integer ' GPS_FIX_QUALITY
            Public FixType As Integer ' GPS_FIX_TYPE
            Public SelectionType As Integer ' GPS_FIX_TYPE
            Public flPositionDilutionOfPrecision As Single
            Public flHorizontalDilutionOfPrecision As Single
            Public flVerticalDilutionOfPrecision As Single
            Public dwSatelliteCount As Integer
            Public rgdwSatellitesUsedPRNs As SatelliteArray
            Public dwSatellitesInView As Integer
            Public rgdwSatellitesInViewPRNs As SatelliteArray
            Public rgdwSatellitesInViewElevation As SatelliteArray
            Public rgdwSatellitesInViewAzimuth As SatelliteArray
            Public rgdwSatellitesInViewSignalToNoiseRatio As SatelliteArray
        End Structure
        Private gpsDeviceHandle As IntPtr = IntPtr.Zero
        Private newPositionHandle As IntPtr = IntPtr.Zero
        Private stopHandle As IntPtr = IntPtr.Zero
        Private eventHandlerThread As Thread = Nothing
        Private brokenStructSizeWM6 As Boolean = False
        Private Const GPS_VALID_ALTITUDE_WRT_SEA_LEVEL As Integer = &H40
        Private Const GPS_VALID_ALTITUDE_WRT_ELLIPSOID As Integer = &H80

        Public Overrides Function Connect() As Boolean
            If gpsDeviceHandle = IntPtr.Zero Then
                stopHandle = CreateEvent(IntPtr.Zero, 0, 0, Nothing)
                newPositionHandle = CreateEvent(IntPtr.Zero, 0, 0, Nothing)
                Try
                    gpsDeviceHandle = GPSOpenDevice(newPositionHandle, Nothing, Nothing, 0)
                Catch ex As Exception
                    ' ignore
                End Try
                Connect = gpsDeviceHandle <> IntPtr.Zero
                If Connect = True Then
                    gpsConnected = True
                    If eventHandlerThread Is Nothing Then
                        eventHandlerThread = New Thread(New ThreadStart(AddressOf Me.IntermediateEventThread))
                        eventHandlerThread.Start()
                    End If
                    RaiseConnected()
                Else
                    gpsConnected = False
                End If
            End If
        End Function
        Public Overrides Sub Disconnect()
            If gpsDeviceHandle <> IntPtr.Zero Then
                GPSCloseDevice(gpsDeviceHandle)
                CloseHandle(gpsDeviceHandle)
                gpsDeviceHandle = IntPtr.Zero
            End If
            If stopHandle <> IntPtr.Zero Then
                SetEvent(stopHandle)
            End If
            gpsConnected = False
            RaiseDisconnected()
        End Sub
        Private Sub IntermediateEventThread()
            Dim abort As Boolean = False
            Dim multiHandler As IntPtr = LocalAlloc(LMEM_ZEROINIT, IntPtr.Size * 2)
            Marshal.WriteInt32(multiHandler, 0, stopHandle.ToInt32())
            Marshal.WriteInt32(multiHandler, IntPtr.Size, newPositionHandle.ToInt32())
            While Not abort
                Dim result As Integer = WaitForMultipleObjects(2, multiHandler, 0, -1)
                If result = 0 Then
                    abort = True
                ElseIf result = 1 Then
                    PollIntermediateDriver()
                End If
            End While
            LocalFree(multiHandler)
            If stopHandle <> IntPtr.Zero Then
                CloseHandle(stopHandle)
                stopHandle = IntPtr.Zero
            End If
            If newPositionHandle <> IntPtr.Zero Then
                CloseHandle(newPositionHandle)
                newPositionHandle = IntPtr.Zero
            End If
            eventHandlerThread = Nothing
        End Sub
        Private Sub PollIntermediateDriver()
            Dim IntermediateGPSPosition As _GPS_POSITION = New _GPS_POSITION

            IntermediateGPSPosition.dwVersion = 1
            ' HACK for http://blogs.msdn.com/cenet/archive/2007/12/06/gpsid-problem-workaround-on-recent-wm6-release.aspx
            If brokenStructSizeWM6 = False Then
                IntermediateGPSPosition.dwSize = Marshal.SizeOf(IntermediateGPSPosition) ' = 344
            Else
                IntermediateGPSPosition.dwSize = 376
            End If

            Dim ptr As IntPtr = LocalAlloc(LMEM_ZEROINIT, IntermediateGPSPosition.dwSize)

            ' check ptr <> IntPtr.Zero otherwise OutOfMemory
            Marshal.StructureToPtr(IntermediateGPSPosition, ptr, False)

            Dim result As Integer = GPSGetPosition(gpsDeviceHandle, ptr, 15000, 0)
            If result = 0 AndAlso gpsConnected = True Then
                IntermediateGPSPosition = CType(Marshal.PtrToStructure(ptr, GetType(_GPS_POSITION)), _GPS_POSITION)

                If IntermediateGPSPosition.FixQuality > 0 Then
                    If HasFix = False Then
                        gpsPosition.FixType = IntermediateGPSPosition.FixQuality
                        RaiseFixChanged()
                    End If
                    If gpsPosition.UTCTime.Second <> IntermediateGPSPosition.stUTCTime.wSecond Then
                        gpsPosition.SetUTCTime(IntermediateGPSPosition.stUTCTime)
                        SyncTime(IntermediateGPSPosition.stUTCTime)
                    End If
                    gpsPosition.Speed = IntermediateGPSPosition.flSpeed
                    gpsPosition.SatelliteCount = IntermediateGPSPosition.dwSatelliteCount
                    ' Altitude
                    If (IntermediateGPSPosition.dwValidFields And GPS_VALID_ALTITUDE_WRT_SEA_LEVEL) <> 0 Then
                        gpsPosition.AltitudeWRTSeaLevel = CInt(IntermediateGPSPosition.flAltitudeWRTSeaLevel)
                    ElseIf (IntermediateGPSPosition.dwValidFields And GPS_VALID_ALTITUDE_WRT_ELLIPSOID) <> 0 Then
                        gpsPosition.AltitudeWRTSeaLevel = CInt(IntermediateGPSPosition.flAltitudeWRTEllipsoid - GPS.Height.WGS84Separation(IntermediateGPSPosition.dblLatitude, IntermediateGPSPosition.dblLongitude))
                    Else
                        gpsPosition.AltitudeWRTSeaLevel = 0
                    End If
                    gpsPosition.PositionDilutionOfPrecision = IntermediateGPSPosition.flPositionDilutionOfPrecision
                    gpsPosition.HorizontalDilutionOfPrecision = IntermediateGPSPosition.flHorizontalDilutionOfPrecision
                    gpsPosition.VerticalDilutionOfPrecision = IntermediateGPSPosition.flVerticalDilutionOfPrecision
                    If Bearing <> IntermediateGPSPosition.flHeading OrElse Latitude <> IntermediateGPSPosition.dblLatitude OrElse Longitude <> IntermediateGPSPosition.dblLongitude Then
                        gpsPosition.Latitude = IntermediateGPSPosition.dblLatitude
                        gpsPosition.Longitude = IntermediateGPSPosition.dblLongitude
                        gpsPosition.Heading = IntermediateGPSPosition.flHeading
                        RaisePositionChanged()
                    End If
                Else
                    If HasFix = True Then
                        gpsPosition.FixType = 0
                        RaiseFixChanged()
                    End If
                End If
                ' TODO: use DeviceStatus for m_LastDataReceived i.e. timeout
                lastDataReceivedDate = Now
            ElseIf result = 87 Then
                brokenStructSizeWM6 = True
            End If
            LocalFree(ptr)
        End Sub
        Protected Overrides Sub Finalize()
            If gpsDeviceHandle <> IntPtr.Zero Then
                GPSCloseDevice(gpsDeviceHandle)
                CloseHandle(gpsDeviceHandle)
                gpsDeviceHandle = IntPtr.Zero
            End If
            MyBase.Finalize()
        End Sub
    End Class
End Namespace
