' Copyright 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Threading
Imports System.Runtime.InteropServices

Namespace GPS
    Public Class NativeSerialPort
        Inherits SerialPort

        <DllImport("coredll.dll", SetLastError:=True)> _
        Private Shared Function CreateFile(ByVal lpFileName As String, ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, ByVal lpSecurityAttributes As Integer, ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As Integer) As IntPtr
        End Function
        <DllImport("coredll.dll", SetLastError:=True)> _
        Private Shared Function ReadFile(ByVal hFile As IntPtr, ByVal byteBuffer() As Byte, ByVal nNumberOfBytesToRead As Integer, ByRef lpNumberOfBytesRead As Integer, ByRef lpOverlapped As Integer) As Integer
        End Function
        Private fileHandle As IntPtr = IntPtr.Zero
        Private readerThread As Thread = Nothing
        Private Const GENERIC_READ As Integer = &H80000000
        Private Const GENERIC_WRITE As Integer = &H40000000
        Private Const FILE_SHARE_READ As Integer = 1
        Private Const FILE_SHARE_WRITE As Integer = 2
        Private Const FILE_ATTRIBUTE_NORMAL As Integer = &H80
        Private Const OPEN_EXISTING As Integer = 3

        'General info: http://msdn.microsoft.com/en-us/library/bb202019.aspx
        'CreateFile: http://msdn.microsoft.com/en-us/library/aa914735.aspx
        'ReadFile: http://msdn.microsoft.com/en-us/library/aa914377.aspx
        Public Sub New(ByRef parser As NMEAParser, ByVal commName As String, ByVal baudrate As Integer)
            MyBase.New(parser, commName, baudrate)
        End Sub
        Public Overrides Function Connect() As Boolean
            If commName = "" Then
                MsgBox("invalid com parameter", MsgBoxStyle.Critical)
                Return False
            End If
            fileHandle = CreateFile(commName & ":", GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, 0, 3, FILE_ATTRIBUTE_NORMAL, 0)
            ' check if open device/file failed
            If fileHandle.ToInt32 = -1 Then
                MsgBox("Error opening com port", MsgBoxStyle.Critical)
                gpsConnected = False
                Return False
            Else
                If readerThread Is Nothing Then
                    readerThread = New Thread(New ThreadStart(AddressOf Me.FileReaderThread))
                    readerThread.Start()
                End If
                MyBase.Connect()
            End If
        End Function
        Public Overrides Sub Disconnect()
            If fileHandle <> IntPtr.Zero Then
                CloseHandle(fileHandle)
                fileHandle = IntPtr.Zero
            End If
            MyBase.Disconnect()
        End Sub
        Private Sub FileReaderThread()
            Dim inputBuffer(29) As Byte ' keep it small for now, because we don't have any timeout for readfile
            Dim oldStr As String = ""
            Dim rc As Integer = -1
            While rc <> 0
                Dim bytesRead As Boolean
                Do
                    bytesRead = False
                    Dim readBytes As Integer = 0
                    rc = ReadFile(fileHandle, inputBuffer, inputBuffer.Length, readBytes, 0)
                    If rc <> 0 AndAlso readBytes > 0 Then
                        bytesRead = True
                        oldStr = oldStr & ArrayToString(inputBuffer, readBytes)
                        Dim feed As Integer = InStr(oldStr, vbLf)
                        If feed > 0 Then
                            Dim str As String = oldStr.Substring(0, feed)
                            LogNMEA(str)
                            ReceivedData(str)
                            oldStr = oldStr.Substring(feed)
                        End If
                    End If
                Loop Until bytesRead = False
            End While
            readerThread = Nothing
        End Sub
        Private Function ArrayToString(ByRef array() As Byte, ByVal len As Integer) As String
            Dim a As String = ""
            For i As Integer = 0 To Math.Min(len - 1, array.Length)
                a &= Chr(array(i))
            Next
            Return a
        End Function
        Protected Overrides Sub Finalize()
            If fileHandle <> IntPtr.Zero Then
                CloseHandle(fileHandle)
                fileHandle = IntPtr.Zero
            End If
            MyBase.Finalize()
        End Sub
    End Class
End Namespace
