' Copyright 2008 Sven Strickroth <email@cs-ware.de>
' based on code by:
' Copyright 2007 Rubke <interrubke@gmail.com>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO

Namespace GPS
    Public Class Simulation
        Inherits GPSDevice

        Private myParser As NMEAParser = Nothing

        ' old values
        Private prevFixState As Boolean
        Private prevLongitude As Double
        Private prevLatitude As Double
        Private prevBearing As Single

        ' simulator
        Private SimTimer As System.Windows.Forms.Timer
        Private simulatorFileName As String = ""
        Private simFile As FileStream
        Private simFileReader As TextReader
        Private timerInterval As Integer = 100 'msec
        Public Property Interval() As Integer
            Get
                Return timerInterval
            End Get
            Set(ByVal value As Integer)
                timerInterval = value
                If SimTimer IsNot Nothing Then
                    SimTimer.Interval = timerInterval
                End If
            End Set
        End Property
        Public Sub New(ByRef parser As NMEAParser, ByVal fileName As String)
            myParser = parser
            simulatorFileName = fileName
        End Sub
        Public Overrides Function Connect() As Boolean
            gpsConnected = True

            simFile = New FileStream(simulatorFileName, FileMode.Open)
            simFileReader = New StreamReader(simFile)

            SimTimer = New System.Windows.Forms.Timer
            AddHandler SimTimer.Tick, AddressOf NMEASimulatorTick
            SimTimer.Interval = timerInterval
            SimTimer.Enabled = True
            RaiseConnected()
            Return Connect
        End Function
        Public Overrides Sub RequestTimeSync()
            MsgBox("Systime sync not possible in simulation mode.")
        End Sub
        Public Overrides Sub Disconnect()
            gpsConnected = False
            If Not SimTimer Is Nothing Then
                SimTimer.Enabled = False
                RemoveHandler SimTimer.Tick, AddressOf NMEASimulatorTick
                SimTimer = Nothing
                simFile.Close()
                simFileReader = Nothing
                simFile = Nothing
            End If
            RaiseDisconnected()
        End Sub
        Private Sub NMEASimulatorTick(ByVal sender As Object, ByVal e As System.EventArgs)
            If Not simFileReader Is Nothing Then
                Dim line As String
                lastDataReceivedDate = Now
                line = simFileReader.ReadLine()
                If line Is Nothing Then
                    simFile.Seek(0, 0) ' restart read
                Else
                    ReceivedData(line)
                End If
            End If
        End Sub
        Private Sub ReceivedData(ByVal sentence As String)
            If myParser.parse(sentence, gpsPosition) = True Then
                If gpsPosition.FixType > 0 Then
                    If prevFixState = False Then
                        prevFixState = True
                        RaiseFixChanged()
                    End If
                    If gpsPosition.Heading <> prevBearing OrElse gpsPosition.Latitude <> prevLatitude OrElse gpsPosition.Longitude <> prevLongitude Then
                        prevLatitude = gpsPosition.Latitude
                        prevLongitude = gpsPosition.Longitude
                        prevBearing = gpsPosition.Heading
                        RaisePositionChanged()
                    End If
                Else
                    If prevFixState = True Then
                        prevFixState = False
                        RaiseFixChanged()
                    End If
                End If
            End If
        End Sub
    End Class
End Namespace
