' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO
Public Class frmFileFolderSelectDialog
    Private fileMode As Boolean = False
    Private selectionfilter As String = "*.*"
    Private Sub PrepareForm()
        InitializeComponent()
        TreeView1.PathSeparator = Path.DirectorySeparatorChar
        TreeView1.Nodes.AddRange(New TreeNode() {New TreeNode(Path.DirectorySeparatorChar)})
        If fileMode Then
            Me.Text = "Select file..."
            menuItem_CreateFolder.Enabled = False
        Else
            Me.Text = "Select folder..."
        End If
        ExpandNode(TreeView1.Nodes.Item(0))
        TreeView1.Nodes.Item(0).Expand()
        Me.Size = Screen.PrimaryScreen.WorkingArea.Size
    End Sub
    Sub New(Optional ByVal fileMode As Boolean = False)
        fileMode = fileMode
        PrepareForm()
    End Sub
    Sub New(ByVal filter As String)
        selectionfilter = filter
        fileMode = True
        PrepareForm()
    End Sub
    Private Sub TreeView1_AfterSelect(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeView1.AfterSelect
        menuItem_OK.Enabled = Not (fileMode And CStr(TreeView1.SelectedNode.Tag) <> "file")
#If Not SmartPhone Then
        Me.ControlBox = menuItem_OK.Enabled
#End If
        menuItem_CreateFolder.Enabled = fileMode = False And CStr(TreeView1.SelectedNode.Tag) <> "file"
        If TreeView1.SelectedNode.IsExpanded = False AndAlso CStr(TreeView1.SelectedNode.Tag) <> "file" Then
            ExpandNode(TreeView1.SelectedNode)
        End If
    End Sub
    Private Sub ExpandNode(ByVal tn As TreeNode)
        Windows.Forms.Cursor.Current = Cursors.WaitCursor
        TreeView1.BeginUpdate()
        tn.Nodes.Clear()
        Dim path As String = GetPathOfNode(tn)
        Dim dataArray As String() = Directory.GetDirectories(path)
        Array.Sort(dataArray)
        For Each subDir As String In dataArray
            Dim newNode As New TreeNode
            Dim dirinfo As New DirectoryInfo(subDir)
            If (dirinfo.Attributes And FileAttributes.Temporary) = FileAttributes.Temporary Then
                newNode.ImageIndex = 1
            Else
                newNode.ImageIndex = 2
            End If
            newNode.SelectedImageIndex = newNode.ImageIndex
            newNode.Text = dirinfo.Name
            tn.Nodes.Add(newNode)
        Next
        If fileMode Then
            dataArray = Directory.GetFiles(path, selectionfilter)
            Array.Sort(dataArray)
            For Each file As String In dataArray
                Dim newNode As New TreeNode
                Dim fileinfo As New FileInfo(file)
                newNode.ImageIndex = 3
                newNode.SelectedImageIndex = newNode.ImageIndex
                newNode.Tag = "file"
                newNode.Text = fileinfo.Name
                tn.Nodes.Add(newNode)
            Next
        End If
        TreeView1.EndUpdate()
        Windows.Forms.Cursor.Current = Cursors.Default
    End Sub
    Private Function GetPathOfNode(ByRef treeNode As TreeNode) As String
        If treeNode.FullPath <> Path.DirectorySeparatorChar Then
            Return treeNode.FullPath.Substring(1)
        Else
            Return Path.DirectorySeparatorChar
        End If
    End Function
    Public Property FileName() As String
        Get
            Return GetPathOfNode(TreeView1.SelectedNode)
        End Get
        Set(ByVal value As String)
            SelectedElement = value
        End Set
    End Property
    Public Property SelectedPath() As String
        Get
            Return GetPathOfNode(TreeView1.SelectedNode)
        End Get
        Set(ByVal value As String)
            SelectedElement = value
        End Set
    End Property
    Public Property SelectedElement() As String
        Get
            Return GetPathOfNode(TreeView1.SelectedNode)
        End Get
        Set(ByVal value As String)
            If value <> "" Then
                Dim isFile As Boolean = False
                Dim fileinfo As New FileInfo(value)
                If File.Exists(value) Then
                    value = fileinfo.DirectoryName
                    isFile = True
                End If
                Dim dirinfo As New DirectoryInfo(value)
                If dirinfo.Exists Then
                    Dim dirparts() As String = Split(value.Substring(1), Path.DirectorySeparatorChar)
                    Dim tn As TreeNode = TreeView1.Nodes.Item(0)
                    For Each part As String In dirparts
                        For Each tn In tn.Nodes
                            If tn.Text = part Then
                                ExpandNode(tn)
                                tn.Expand()
                                TreeView1.SelectedNode = tn
                                Exit For
                            End If
                        Next
                    Next
                    If fileMode Then
                        If isFile Then
                            For Each tn In tn.Nodes
                                If tn.Text = fileinfo.Name Then
                                    TreeView1.SelectedNode = tn
                                    Exit For
                                End If
                            Next
                        End If
                    Else
                        tn.Toggle()
                    End If
                End If
            End If
        End Set
    End Property
    Public ReadOnly Property Filter() As String
        Get
            Return selectionfilter
        End Get
    End Property
    Private Sub MenuItem_Cancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItem_Cancel.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub
    Private Sub MenuItem_CreateFolder_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItem_CreateFolder.Click
        If CStr(TreeView1.SelectedNode.Tag) <> "file" Then
            Dim newName As String = InputBox("Enter a foldername for a new folder", "Create new Folder", "New Folder")
            If newName <> "" Then
                Try
                    Directory.CreateDirectory(GetPathOfNode(TreeView1.SelectedNode) & Path.DirectorySeparatorChar & newName)
                    ExpandNode(TreeView1.SelectedNode)
                Catch ex As Exception
                    MsgBox("Could not create directory: " & ex.Message)
                End Try
            End If
        End If
    End Sub
    Private Sub TreeView1_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TreeView1.KeyPress
        If Not (fileMode And CStr(TreeView1.SelectedNode.Tag) <> "file") = True And e.KeyChar = Chr(13) Then
            Me.DialogResult = Windows.Forms.DialogResult.OK
            Me.Close()
        End If
    End Sub
    Private Sub menuItem_OK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles menuItem_OK.Click
        If Not (fileMode And CStr(TreeView1.SelectedNode.Tag) <> "file") = True Then
            Me.DialogResult = Windows.Forms.DialogResult.OK
            Me.Close()
        End If
    End Sub
End Class
