' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO
Imports System.Collections.Generic

''' <summary>
''' Class for general handling of a GPX file.
''' </summary>
Public MustInherit Class GPXFile
    Public Shared myCultureInfo As New System.Globalization.CultureInfo("en-US")
    Public Shared GPX_EXTENSION As String = "gpx"
    Public Class OSMTrackPoint
        Public Position As MAP_POSITION
        Public tilePosX As Integer
        Public tilePosY As Integer
        Public skipPaint As Boolean = False
        Public Sub New(ByVal position As MAP_POSITION)
            Me.Position = position
        End Sub
        Public Function getLongitudeString() As String
            getLongitudeString = Position.LonF.ToString("0.0000000", myCultureInfo)
        End Function
        Public Function getLatitudeString() As String
            getLatitudeString = Position.LatF.ToString("0.0000000", myCultureInfo)
        End Function
        Public Sub SetCache(ByVal v1 As Integer, ByVal v2 As Integer, ByVal v3 As Boolean)
            tilePosX = v1
            tilePosY = v2
            skipPaint = v3
        End Sub
    End Class
    Public Structure OSMWayPoint
        Public Latitude As Double
        Public Longitude As Double
        Public Ele As Integer
        Public Time As String
        Public Name As String
        Public Comment As String
        Public LinkHref As String
        Public PositionDilutionOfPrecision As Single
        Public HorizontalDilutionOfPrecision As Single
        Public VerticalDilutionOfPrecision As Single
        Public Sub New(ByVal position As GPS_POSITION, ByVal time As Date, ByVal name As String, ByVal comment As String, ByVal linkHref As String)
            Me.Latitude = position.Latitude
            Me.Longitude = position.Longitude
            Me.Time = time.ToUniversalTime.ToString("yyyy-MM-ddTHH:mm:ssZ", myCultureInfo)
            Me.Name = name
            Me.Comment = comment
            Me.LinkHref = linkHref
            Me.Ele = position.AltitudeWRTSeaLevel
            Me.PositionDilutionOfPrecision = position.PositionDilutionOfPrecision
            Me.HorizontalDilutionOfPrecision = position.HorizontalDilutionOfPrecision
            Me.VerticalDilutionOfPrecision = position.VerticalDilutionOfPrecision
        End Sub
        Public Function GetLongitudeString() As String
            Return Longitude.ToString("0.0000000", myCultureInfo)
        End Function
        Public Function GetLatitudeString() As String
            Return Latitude.ToString("0.0000000", myCultureInfo)
        End Function
        Public Function GetPosition() As MAP_POSITION
            Return New MAP_POSITION(Latitude, Longitude)
        End Function
        Public Function ToXML(ByVal gpxWithDoP As Boolean) As String
            ToXML = "<wpt lat=""" & GetLatitudeString() & """ lon=""" & GetLongitudeString() & """><ele>" & Ele & "</ele><time>" & Time & "</time><name>" & Encode(Name) & "</name>"
            If LinkHref.Length > 0 Then
                ToXML &= "<link href=""" & LinkHref & """ />"
            End If
            If gpxWithDoP Then
                ToXML &= "<hdop>" & HorizontalDilutionOfPrecision.ToString(myCultureInfo) & "</hdop><vdop>" & VerticalDilutionOfPrecision.ToString(myCultureInfo) & "</vdop><pdop>" & PositionDilutionOfPrecision.ToString(myCultureInfo) & "</pdop>"
            End If
            ToXML &= "</wpt>" & vbCrLf
        End Function
    End Structure
    Public waypoint_mutex As New Threading.Mutex ' HACK: not nice to have this public
    ''' <summary>
    ''' Adds and starts a new track in the GPX file.
    ''' </summary>
    Public MustOverride Sub AddTrack()
    ''' <summary>
    ''' Adds a new waypoint (Point Of Interest) to the GPX file.
    ''' </summary>
    ''' <param name="position">The position.</param>
    ''' <param name="time">The time.</param>
    ''' <param name="name">The name.</param>
    ''' <param name="comment">The comment.</param>
    ''' <param name="linkhref">The linkhref.</param>
    Public MustOverride Sub AddWayPoint(ByRef position As GPS_POSITION, ByVal time As Date, ByVal name As String, ByVal comment As String, ByVal linkhref As String)
    ''' <summary>
    ''' Adds a trackpoint to the current track of the GPX file.
    ''' </summary>
    ''' <param name="position">The position.</param>
    ''' <param name="time">The time.</param>
    Public MustOverride Sub AddTrackPoint(ByVal position As GPS_POSITION, ByVal time As Date)
    ''' <summary>
    ''' Saves the opened GPX file and closes it.
    ''' </summary>
    Public MustOverride Sub Save()
    ''' <summary>
    ''' Returns the number of waypoints.
    ''' </summary>
    ''' <returns></returns>
    Public MustOverride Function GetWayPointsCount() As Integer
    ''' <summary>
    ''' Returns the number track points of the current track.
    ''' </summary>
    ''' <returns></returns>
    Public MustOverride Function GetTrackPointsCount() As Integer
    ''' <summary>
    ''' Removes the waypoint at position index.
    ''' </summary>
    ''' <param name="index">The index.</param>
    Public MustOverride Function RemoveWayPoint(ByVal index As Integer) As Boolean
    ''' <summary>
    ''' Returns the waypoints as a LinkedList.
    ''' </summary>
    ''' <returns></returns>
    Public MustOverride Function GetWayPoints() As SingleLinkedList(Of OSMWayPoint)
    ''' <summary>
    ''' Returns the trackpoints of the current track as a LinkedList.
    ''' </summary>
    ''' <returns></returns>
    Public MustOverride Function GetTrackPoints() As SingleLinkedList(Of OSMTrackPoint)
End Class
