' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Public Class Form1
    Friend WithEvents logic As OSMtrackerLogic
    Public Sub New()
        frmSplashScreen.Show()
        Application.DoEvents()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        logic = New OSMtrackerLogic(Me, OsmTileMap1, picTitleBar)
        logic.AfterCreationSetup()

        Application.DoEvents()
        frmSplashScreen.Close()
    End Sub
    Private Sub UpdateGPSpanel(ByVal myPos As GPS_POSITION) Handles logic.UpdateGPSInformation
        With frmGPSView
            .lblInfo1.Text = "Pos: " & myPos.Latitude.ToString("0.000000") & "  " & myPos.Longitude.ToString("0.000000")
            .lblInfo2.Text = "Alt: " & myPos.AltitudeWRTSeaLevel & " Spd: " & logic.GPSDevice.Speed.ToString("0.0") & " Sat: " & myPos.SatelliteCount & " " & logic.GPSDevice.FixTypeString
            .lblInfo3.Text = "HVP dop: " & myPos.HorizontalDilutionOfPrecision.ToString("0.0") & "  " & myPos.VerticalDilutionOfPrecision.ToString("0.0") & "  " & myPos.PositionDilutionOfPrecision.ToString("0.0")
            If logic.HasFix AndAlso myPos.UTCTime.Year > 0 Then
                .lblInfo4.Text = "Time: " & myPos.UTCTime.ToLocalTime.ToShortDateString & " " & myPos.UTCTime.ToLocalTime.ToLongTimeString
            Else
                .lblInfo4.Text = "Time not available"
            End If
        End With
    End Sub
    Private Sub UpdateButtons(ByVal myPos As GPS_POSITION) Handles logic.UpdateGUIButtons
        mnuConnect.Enabled = Not logic.IsConnected
        mnuDisconnect.Enabled = Not mnuConnect.Enabled
        mnuGPSsyncTime.Enabled = logic.HasFix

        If frmButtonView.Visible = True Then
            frmButtonView.ButtonPanel1.SetupPage(logic.HasFix)
        End If
    End Sub
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub
    Private Sub mnuConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuConnect.Click
        logic.Connect()
    End Sub
    Private Sub mnuDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDisconnect.Click
        logic.Disconnect()
    End Sub
    Private Sub logic_UpdateGUIByViewTimer(ByVal position As GPS_POSITION) Handles logic.UpdateGUIByViewTimer
        If logic.IsConnected AndAlso logic.HasFix Then
            frmGPSView.SpeedGraph.AddPoint(CInt(logic.GPSDevice.Speed), frmGPSView.Visible)
        End If
        picTitleBar.UpdateData(logic.HasFix, logic.IsConnected, position.SatelliteCount, logic.GPSDevice.Speed, logic.GPSDevice.Bearing, logic.TrackLogging)
    End Sub
    Private Sub logic_UpdateGPSPosOnMap(ByVal position As GPS_POSITION) Handles logic.UpdateGPSPosOnMap
        OsmTileMap1.SetGPSPosition(position, Not (frmButtonView.Visible Or frmGPSView.Visible Or frmPOITableView.Visible))
    End Sub
    Sub FillPosListView() Handles logic.UpdateDataList
        If frmPOITableView.Visible Then
            frmPOITableView.FillPosListView()
        End If
    End Sub
    Private Sub mnuGPSsyncTime_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuGPSsyncTime.Click
        logic.SyncTime()
    End Sub
    Private Sub mnuGPSsim_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuGPSsim.Click
        logic.StartSimulation()
    End Sub
    Private Sub mnuAbout2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAbout.Click
        frmAbout.ShowDialog()
    End Sub
    Private Sub frmMain_Down(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        logic.HandleKeyPress(e)
    End Sub
    Private Sub frmMain_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        logic.ShuttingDown()
    End Sub
    Private Sub mnuOpenGPX_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuOpenGPX.Click
        Dim dialog As frmFileFolderSelectDialog = New frmFileFolderSelectDialog("*." & GPXFile.GPX_EXTENSION)
        With dialog
            .SelectedElement = logic.DataFolder
            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                logic.OpenRouteGPXFile(.SelectedElement)
            End If
        End With
    End Sub
    Private Sub mnuNewGPX_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuNewGPX.Click
        logic.NewGPXFile()
    End Sub
    Private Sub mnuSettingsGeneral_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettingsGeneral.Click
        logic.ShowSettingsGeneral()
    End Sub
    Private Sub mnuSettingsGPS_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettingsGPS.Click
        logic.ShowSettingsGPS()
    End Sub
    Private Sub mnuSettingsVoice_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettingsVoice.Click
        logic.ShowSettingsVoice()
    End Sub
    Private Sub mnuSettingsLogging_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettingsLogging.Click
        logic.ShowSettingsLogging()
    End Sub
    Private Sub mnuSettingsMap_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettingsMap.Click
        logic.ShowSettingsMap()
    End Sub
    Private Sub mnuViewsButtons_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuViewsButtons.Click
        frmButtonView.ButtonPanel1.SetupPage(logic.HasFix)
        frmButtonView.Controls.Add(picTitleBar)
        frmButtonView.ShowDialog()
        Me.Controls.Add(picTitleBar)
    End Sub
    Private Sub mnuViewsGPS_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuViewsGPS.Click
        frmGPSView.SpeedGraph.UpdateSpeedGraph(True)
        frmGPSView.Controls.Add(picTitleBar)
        frmGPSView.ShowDialog()
        Me.Controls.Add(picTitleBar)
    End Sub
    Private Sub mnuViewsData_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuViewsData.Click
        frmPOITableView.FillPosListView()
        frmPOITableView.Controls.Add(picTitleBar)
        frmPOITableView.ShowDialog()
        Me.Controls.Add(picTitleBar)
    End Sub
    Private Sub logic_UpdateSimulation(ByVal enabled As Boolean) Handles logic.UpdateSimulation
        mnuGPSsim.Enabled = enabled
    End Sub
End Class
