' Copyright 2009-2010 Jozef Vince <jozef.vince@freemap.sk>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Runtime.InteropServices

Public Class WebCamera
    Private bmpOffscreen As Bitmap

    Const WM_CAP As Short = &H400S

    Const WM_CAP_DRIVER_CONNECT As Integer = WM_CAP + 10
    Const WM_CAP_DRIVER_DISCONNECT As Integer = WM_CAP + 11
    Const WM_CAP_EDIT_COPY As Integer = WM_CAP + 30

    Const WM_CAP_SET_PREVIEW As Integer = WM_CAP + 50
    Const WM_CAP_SET_PREVIEWRATE As Integer = WM_CAP + 52
    Const WM_CAP_SET_SCALE As Integer = WM_CAP + 53
    Const WS_CHILD As Integer = &H40000000
    Const WS_VISIBLE As Integer = &H10000000
    Const SWP_NOMOVE As Short = &H2S
    Const SWP_NOSIZE As Short = 1
    Const SWP_NOZORDER As Short = &H4S
    Const HWND_BOTTOM As Short = 1

    Const WM_CAP_DLG_VIDEOFORMAT As Integer = WM_CAP + 41
    Const WM_CAP_DLG_VIDEOSOURCE As Integer = WM_CAP + 42
    Const WM_CAP_DLG_VIDEODISPLAY As Integer = WM_CAP + 43
    Const WM_CAP_GET_VIDEOFORMAT As Integer = WM_CAP + 44
    Const WM_CAP_SET_VIDEOFORMAT As Integer = WM_CAP + 45
    Const WM_CAP_DLG_VIDEOCOMPRESSION As Integer = WM_CAP + 46

    Const WM_CAP_SET_OVERLAY As Integer = WM_CAP + 51
    Const WM_CAP_GRAB_FRAME_NOSTOP As Integer = WM_CAP + 61
    Const WM_CAP_SET_CALLBACK_FRAME As Integer = WM_CAP + 5
    Const WM_CAP_SAVEDIB As Integer = WM_CAP + 25

    Dim iDevice As Integer = 0 ' Current device ID
    Dim hHwnd As Integer ' Handle to preview window

    Dim myMessage As String

    Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
        (ByVal hwnd As Integer, ByVal wMsg As Integer, ByVal wParam As Integer, _
        <MarshalAs(UnmanagedType.AsAny)> ByVal lParam As Object) As Integer

    Private Declare Function SetWindowPos Lib "user32" Alias "SetWindowPos" (ByVal hwnd As Integer, _
        ByVal hWndInsertAfter As Integer, ByVal x As Integer, ByVal y As Integer, _
        ByVal cx As Integer, ByVal cy As Integer, ByVal wFlags As Integer) As Integer

    Private Declare Function DestroyWindow Lib "user32" (ByVal hndw As Integer) As Boolean

    Private Declare Function capCreateCaptureWindowA Lib "avicap32.dll" _
        (ByVal lpszWindowName As String, ByVal dwStyle As Integer, _
        ByVal x As Integer, ByVal y As Integer, ByVal nWidth As Integer, _
        ByVal nHeight As Short, ByVal hWndParent As Integer, _
        ByVal nID As Integer) As Integer

    Private Declare Function capGetDriverDescriptionA Lib "avicap32.dll" (ByVal wDriver As Short, _
        ByVal lpszName As String, ByVal cbName As Integer, ByVal lpszVer As String, _
        ByVal cbVer As Integer) As Boolean

    Protected Overrides Sub OnPaintBackground(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaintBackground(pe)
        ' needed!!!  otherwise the speedgraph might flash
    End Sub

    Private previewOn As Boolean = False

    Public Property IsPreviewActive() As Boolean
        Get
            Return previewOn
        End Get
        Set(ByVal value As Boolean)
            If previewOn <> value Then
                If previewOn = False Then
                    OpenPreviewWindow()
                Else
                    ClosePreviewWindow()
                End If
            End If
            previewOn = value
        End Set
    End Property
    Public Function CaptureStillImage(ByVal imageName As String) As Boolean
        If previewOn = True Then
            Dim data As IDataObject
            Dim bmap As Image

            ' Copy image to clipboard
            SendMessage(hHwnd, WM_CAP_EDIT_COPY, 0, 0)

            ' Get image from clipboard and convert it to a bitmap
            data = Clipboard.GetDataObject()
            If data.GetDataPresent(GetType(System.Drawing.Bitmap)) Then
                bmap = CType(data.GetData(GetType(System.Drawing.Bitmap)), Image)
                Me.Image = bmap
                bmap.Save(imageName, Imaging.ImageFormat.Png)
                Return (True)
            End If
        Else
            Return (False)
        End If
    End Function
    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        Dim gxOff As Graphics '      //Offscreen graphics
        Dim rc As Rectangle = Me.ClientRectangle

        If (bmpOffscreen Is Nothing) Then '//Bitmap for doublebuffering
            bmpOffscreen = New Bitmap(ClientSize.Width, ClientSize.Height, Imaging.PixelFormat.Format16bppRgb565)
        End If

        gxOff = Graphics.FromImage(bmpOffscreen)
        gxOff.Clear(Me.BackColor)

        ' border
        'rc.Width -= 1
        'rc.Height -= 1
        'gxOff.DrawRectangle(New Pen(Color.Black, 1), rc)

        gxOff.DrawString(myMessage, New Font("Arial", 8), Brushes.White, 1, Me.Height - 16)
        'Draw from the memory bitmap
        pe.Graphics.DrawImage(bmpOffscreen, 0, 0)
        MyBase.OnPaint(pe)
    End Sub
    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ShowMessage("Preview OFF")
        Me.SizeMode = PictureBoxSizeMode.StretchImage
    End Sub

    Public Sub LoadDeviceList()
        Dim strName As String = Space(100)
        Dim strVer As String = Space(100)
        Dim bReturn As Boolean
        Dim x As Short = 0

        ' Load name of all avialable devices into the lstDevices
        Do
            '   Get Driver name and version
            bReturn = capGetDriverDescriptionA(x, strName, 100, strVer, 100)

            ' If there was a device add device name to the list
            If bReturn Then
                'lstDevices.Items.Add(strName.Trim)
            End If
            x += 1S
        Loop Until bReturn = False
    End Sub

    Public Sub OpenPreviewWindow()
        Dim iHeight As Integer = Me.Height
        Dim iWidth As Integer = Me.Width

        ' Open Preview window in picturebox
        hHwnd = capCreateCaptureWindowA("Device " & iDevice.ToString(), WS_VISIBLE Or WS_CHILD, 0, 0, 640, 480, Me.Handle.ToInt32, 0)

        ' Connect to device
        If SendMessage(hHwnd, WM_CAP_DRIVER_CONNECT, iDevice, 0) <> 0 Then

            SendMessage(hHwnd, WM_CAP_SET_OVERLAY, 1, 0)

            'Set the preview scale
            SendMessage(hHwnd, WM_CAP_SET_SCALE, 1, 0)

            'Set the preview rate in milliseconds
            SendMessage(hHwnd, WM_CAP_SET_PREVIEWRATE, 66, 0)

            'Start previewing the image from the camera
            SendMessage(hHwnd, WM_CAP_SET_PREVIEW, 1, 0)

            ' Resize window to fit in picturebox
            SetWindowPos(hHwnd, HWND_BOTTOM, 0, 0, Me.Width, Me.Height - 16, SWP_NOMOVE Or SWP_NOZORDER)

            previewOn = True
            ShowMessage("Preview ON")
        Else
            ' Error connecting to device close window
            previewOn = False
            DestroyWindow(hHwnd)
            ShowMessage("Preview ERROR")
        End If
    End Sub
    Public Sub ClosePreviewWindow()
        ' Disconnect from device
        SendMessage(hHwnd, WM_CAP_DRIVER_DISCONNECT, iDevice, 0)

        ' close window
        DestroyWindow(hHwnd)
        previewOn = False
        ShowMessage("Preview OFF")
    End Sub
    Public Sub ShowVideoFormatDialog()
        If previewOn = True Then
            SendMessage(hHwnd, WM_CAP_DLG_VIDEOFORMAT, 0, 0)
        End If
    End Sub
    Protected Overrides Sub Finalize()
        If previewOn = True Then
            ClosePreviewWindow()
        End If
        MyBase.Finalize()
    End Sub
    Private Sub ShowMessage(ByVal message As String)
        myMessage = message
        Me.Invalidate()
    End Sub
End Class
