' Copyright 2009 Jean-Pierre Cadiat <csu333-osmtracker@yahoo.fr>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

''' <summary>
''' Class for showing a progress bar.
''' </summary>
Public Class Progress
    Private subSect As New Stack(Of Integer)
    Private part As Single
    Private sumOfParts As Double
    Private start As Date
    Private aborted As Boolean = False
    Private Sub Progress_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Left = CInt((Screen.PrimaryScreen.WorkingArea.Width - Width) / 2)
        Top = CInt((Screen.PrimaryScreen.WorkingArea.Height - Height) * 0.2)
        start = Date.Now
        ProgressBar1.Value = 0
        sumOfParts = 0
        part = 100
    End Sub
    ''' <summary>
    ''' Tells the progress bar the walker enter a new subdirectory.
    ''' </summary>
    ''' <param name="subSections">the number of directory in the new subdirectory</param>
    Public Sub IncreaseDepth(ByVal subSections As Integer)
        subSect.Push(subSections)
        If subSections > 0 Then
            part = part / subSections
        End If
    End Sub
    ''' <summary>
    ''' Tells the progress bar the walker go out of current subdirectory.
    ''' </summary>
    Public Sub DecreaseDepth()
        Dim tmp As Integer = subSect.Pop()
        If tmp > 0 Then
            part = part * tmp
        End If
        If subSect.Count = 0 Then
            start = Date.Now
            sumOfParts = 0
            part = 100
        End If
    End Sub
    ''' <summary>
    ''' Return whether walker reached the last directory of this subdirectory or not.
    ''' </summary>
    ''' <returns></returns>
    Public Function IsLast() As Boolean
        Return subSect.Count = 0
    End Function
    ''' <summary>
    ''' Tells the progress bar that the walker moves to the next directory.
    ''' </summary>
    Public Sub Increase()
        sumOfParts = sumOfParts + part
        If (sumOfParts > 100) Then sumOfParts = 100
        ProgressBar1.Value = CInt(sumOfParts)
        lblElapsed.Text = "Elapsed: " & New TimeSpan((Date.Now - start).Ticks).ToString()
        If lblElapsed.Text.Length > 18 Then
            lblElapsed.Text = lblElapsed.Text.Substring(0, 18)
        End If
        If Date.Now > start Then
            Dim ticks As Long = (Date.Now - start).Ticks
            lblRemaining.Text = "Remaining: " & New TimeSpan(CLng((ticks / (sumOfParts / 100) - ticks))).ToString()
        End If
        If lblRemaining.Text.Length > 20 Then
            lblRemaining.Text = lblRemaining.Text.Substring(0, 20)
        End If
        Application.DoEvents()
    End Sub
    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)
        e.Graphics.DrawRectangle(New Pen(Color.Black, 2), 0, 0, Width - 1, Height - 1)
    End Sub
    Public ReadOnly Property IsAborted() As Boolean
        Get
            Return aborted
        End Get
    End Property
    Private Sub abortButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles abortButton.Click
        abortButton.Enabled = False
        aborted = True
    End Sub
End Class